package org.zheng.demo;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.Arrays;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

/*
 * Covers Declaration, Creation, Initialization, Accessing, Conversation, comparison, copy, fill, etc
 */

class TestArraysBasic {

	// Array Declaration,
	// declare an array by specifying the type of its elements followed by square
	// brackets
	private String[] stringArray;
	private Integer[] intArray;
	private boolean[] booleanArray;

	// Declaration, Creation, Initialization
	boolean[] booleanArray2 = { true, false, true };

	@BeforeEach
	void setup() {
		// Array Creation
		// create an array using the new keyword followed by the type of elements and
		// the number of elements
		stringArray = new String[] { "Mary", "John", "Bob" };
		booleanArray = new boolean[] { true, false, true };
		intArray = new Integer[] { 1, 3, 5 };

	}

	@Test
	void test_asList() {
		List<String> stringList = Arrays.asList(stringArray);

		assertEquals("Mary", stringList.get(0), "the first List is Mary");
		assertEquals("John", stringList.get(1), "the second List is John");
		assertEquals("Bob", stringList.get(2), "the third List is Bob");

		// changes made to array is cascaded to the list
		stringArray[0] = "Zheng";
		assertEquals("Zheng", stringList.get(0), "the first List is Changed to Zheng");

		// changes made to list is cascaded to the array as well
		stringList.set(2, "Terry");
		assertEquals("Terry", stringArray[2], "the third Array is Change to Terry");

	}

	@Test
	void test_compare() {
		int return0 = Arrays.compare(booleanArray, booleanArray2);
		assertEquals(0, return0, "both array should be same");

		// Accessing Array Elements
		booleanArray2[2] = false;
		booleanArray2[1] = true;
		int return1 = Arrays.compare(booleanArray2, booleanArray);
		assertEquals(1, return1, "both array should NOT be same after swapped order");
	}

	@Test
	void test_copyOf() {
		boolean[] newArray = Arrays.copyOf(booleanArray, 5);
		assertEquals(5, newArray.length);
		assertFalse(newArray[4], "extra elements use default false value");

		newArray[3] = true;
		assertTrue(newArray[3], "updated element to true");
	}

	@Test
	void test_fill() {
		String[] newArray = Arrays.copyOf(stringArray, 20);

		Arrays.fill(newArray, 0, 19, "NA");

		List<String> tt = Arrays.asList(newArray);

		tt.stream().limit(19).forEach(boo -> assertTrue(boo.equalsIgnoreCase("NA")));
		assertNull(newArray[19]);
	}

	@Test
	void test_mismatch() {
		String[] newArray = Arrays.copyOf(stringArray, 4);

		int mismatchIndex = Arrays.mismatch(stringArray, newArray);
		assertEquals(3, mismatchIndex, "this first mismatch index is 3");
	}

	@Test
	void test_toString() {
		String booleanArrayString = Arrays.toString(booleanArray);
		assertEquals("[true, false, true]", booleanArrayString, "the boolean [] toString");
		System.out.println("booleanArrayString: " + booleanArrayString.toString());

		String intArrayString = Arrays.toString(intArray);
		assertEquals("[1, 3, 5]", intArrayString, "the Integer [] toString");
		System.out.println("intArrayString: " + intArrayString.toString());

		// but ToString on String is the memory address
		assertTrue(stringArray.toString().contains("java.lang.String"));
		System.out.println("stringArray: " + stringArray.toString());

	}
}
